jQuery.sap.declare("sas.risk.mrm.biz.Link");

sas.risk.mrm.biz.Link=function(businessObject1,businessObject2,changeReason,creator,description,key,linkInstanceId,linkType,sourceSystemCd,customFields,sourceKey,lastModifiedDttm,links,isDisabled){
	this.businessObject1=businessObject1;
	this.businessObject2=businessObject2;
	this.changeReason=changeReason;
	this.creator=creator;
	this.description=description;
	this.key=key;
	this.linkInstanceId=linkInstanceId;
	this.linkType=linkType;
	this.sourceSystemCd=sourceSystemCd;
	this.customFields=customFields;
	this.sourceKey=sourceKey;
	this.targetJson=null;
	this.pageContext=null;
	this.lastModifiedDttm=lastModifiedDttm;
	this.linkInstanceLinks=links;
	this.isDisabled=isDisabled;
	this.validFrom=null;
};
sas.risk.mrm.biz.Link.prototype={
	getType:function(){
		return this.linkType;
	},
	getBusinessObject1:function(){
		return this.businessObject1;
	},
	getBusinessObject2:function(){
		return this.businessObject2;
	},
	getTargetKey:function(){
		return (!this.businessObject1||this.businessObject1===this.sourceKey)?this.businessObject2:this.businessObject1;
	},
	setSourceKey:function(sourceKey){
		this.sourceKey=sourceKey;
	},
	getSourceKey:function(){
		return this.sourceKey;
	},
	getIsDisabled:function(){
	   return this.isDisabled;
	},
	hasTargetJson:function(){
		return !!this.targetJson;
	},
	setTargetJson:function(json){
		this.targetJson=json;
	},
	getTargetJson:function(){
		return this.targetJson;
	},
	getLinkContainerJsonRep:function(){
		var json={
			businessObject1:this.businessObject1,
			businessObject2:this.businessObject2,
			changeReason:this.changeReason,
			creator:this.creator,
			description:this.description,
			key:this.key,
			linkInstanceId:this.linkInstanceId,
			linkType:this.linkType,
			sourceSystemCd:this.sourceSystemCd
		}
		if(this.customFields){
			json.customFields=this.customFields;
		}
		if(this.validFrom){
		   json.lastModifiedDttm=this.validFrom;
		}
		return json;
	},
	getJsonRep:function(){
		var json={
			linkInstanceCreator:this.creator,
			linkInstanceId:this.linkInstanceId,
			linkInstanceLastModifiedDt:this.lastModifiedDttm,
			linkInstanceSourceSystemCd:this.sourceSystemCd,
			linkInstanceDesc:this.description,
			linkInstanceLinks:this.linkInstanceLinks
		};
		if(this.customFields){
			json.customFields=$.extend({},this.customFields);
		}
		return $.extend(true,json,this.targetJson);
	},
	setPageContext:function(pageContext){
		this.pageContext=pageContext;
	},
	getPageContext:function(){
		return this.pageContext;
	},
	getKey:function(){
		return this.key;
	},
	setKey:function(key){
		this.key=key;
	},
	setLinkInstanceId:function(id){
		this.linkInstanceId=id;
	},
	setDescription:function(description){
		this.description=description;
	},
	setCustomFields:function(customFields){
		this.customFields=customFields;
	},
    setValidFrom:function(validFrom){
       this.validFrom=validFrom;
   },	
};
//STATICS
sas.risk.mrm.biz.Link.fromJson=function(linkJson){
	return new sas.risk.mrm.biz.Link(
		linkJson.businessObject1,
		linkJson.businessObject2,
		linkJson.changeReason,
		linkJson.creator,
		linkJson.description,
		linkJson.key,
		linkJson.linkInstanceId,
		linkJson.linkType,
		linkJson.sourceSystemCd,
		linkJson.customFields,
		linkJson.sourceKey,
		linkJson.lastModifiedDttm,
		linkJson.links,
		linkJson.isDisabled
	);
};
sas.risk.mrm.biz.Link.findByType=function(linkContainer,type,sourceKey,sideOfLink){
	var links={};
	if(!linkContainer){return links;}
	var i,json,link;
	for(i=0;i<linkContainer.length;i++){
		json=linkContainer[i];
		if(sas.risk.mrm.biz.Link._matches(json,type,sourceKey,sideOfLink)){
			link=new sas.risk.mrm.biz.Link(json.businessObject1,json.businessObject2,json.changeReason,json.creator,json.description,
					json.key,json.linkInstanceId,json.linkType,json.sourceSystemCd,json.customFields,sourceKey,json.lastModifiedDttm,json.links,json.isDisabled);
			links[link.getTargetKey().toString()]=link;
		}
	}
	return links;
};
sas.risk.mrm.biz.Link._matches=function(json,type,sourceKey,sideOfLink){
	if(json.linkType!==type){return false;}
	if(sideOfLink!==null){//enforce side
		var matchBizObj=(!sideOfLink)?json.businessObject1:json.businessObject2;
		if(json.isDisabled && matchBizObj){
			return false;//disabled must have empty/falsy bizObj1
		}				
		if(!json.isDisabled && sourceKey!==matchBizObj){
			return false;//non-disabled must match bizObj1
		}
	}else{//do not enforce side, either side is a match
		if(json.isDisabled && json.businessObject1 && json.businessObject2){
			return false;//disabled must have empty/falsy on one of the two sides
		}		
		if(!json.isDisabled && sourceKey!==json.businessObject1 && sourceKey!==json.businessObject2){
			return false;//non-disabled must match one of the two sides 
		}
	}
	return true;
};
sas.risk.mrm.biz.Link.matchesLink=function(newLink,oldLink,sourceKey){
	return newLink.businessObject1===oldLink.businessObject1
		&& newLink.businessObject2===oldLink.businessObject2
		&& newLink.linkType===oldLink.linkType
		&& sourceKey===oldLink.sourceKey;
};
sas.risk.mrm.biz.Link.updateLinks=function(newLinkContainer,existingLinks,sourceKey){
	_.each(newLinkContainer,function(json){
		var tempLink=new sas.risk.mrm.biz.Link(json.businessObject1,json.businessObject2,json.changeReason,json.creator,json.description,
				json.key,json.linkInstanceId,json.linkType,json.sourceSystemCd,json.customFields,sourceKey,json.lastModifiedDttm,json.links,json.isDisabled);
		var targetKey=tempLink.getTargetKey();
		var existingLink=existingLinks[targetKey.toString()];
		if(existingLink){
			if(sas.risk.mrm.biz.Link.matchesLink(tempLink, existingLink, sourceKey)){
				sas.risk.mrm.biz.Link.updateLink(tempLink, existingLink);
			}
		}
	});
	return existingLinks;
};
sas.risk.mrm.biz.Link.updateLink=function(newLink,oldLink){
	oldLink.setKey(newLink.key);
	oldLink.setLinkInstanceId(newLink.linkInstanceId);
};