jQuery.sap.require("sas.risk.mrm.components.DimensionBrowserComponent");
jQuery.sap.require("sas.ui.cpb3.components.AComponentMixin");
jQuery.sap.require("sas.ui.cpb3.WidgetBindingUtil");
jQuery.sap.require("sap.ui.commons.Splitter");
jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.hc.m.MenuButton");
jQuery.sap.require("sas.hc.m.Menu");
jQuery.sap.require("sas.hc.m.MenuItem");
jQuery.sap.require("sas.risk.mrm.util.MRMUtil");
jQuery.sap.require("sas.ui.cpb3.nav.NavUtil");
jQuery.sap.require("sas.risk.mrm.components.DimensionNodeComponent");
jQuery.sap.require("sas.risk.mrm.biz.DimensionDefinition");
jQuery.sap.require("sas.risk.mrm.biz.DimNode");
jQuery.sap.require("sas.risk.mrm.biz.DimNodeRegistry");
jQuery.sap.require("sas.risk.mrm.api.MRMCpbObjectAPI");
jQuery.sap.require("sas.risk.mrm.data.MRMObjectModel");
jQuery.sap.require("sas.ui.cpb3.api.CpbQueryAPI");
jQuery.sap.require("sas.risk.common.widgets.Button");

sas.risk.common.layout.flexlayout.FlexLayout.extend("sas.risk.mrm.components.DimensionBrowserComponent", sas.ui.cpb3.components.AComponentMixin.mixinMeta({
	metadata : {
		properties : {
			meta:{type:"any"}
		}
	},
	constructor : function() {
		this.supportsLayout=true;
		this.supportsLabel=true;
		this.supportsAlign=true;
		sas.risk.common.layout.flexlayout.FlexLayout.apply(this,arguments);
		var flexLayoutData=new sas.risk.common.layout.flexlayout.FlexLayoutData({
			alignSelf:sas.risk.common.layout.flexlayout.FlexLayoutDataAlignSelf.Stretch,
			flexGrow:2
		});
		this.setLayoutData(flexLayoutData);
	},
	renderer:"sas.risk.common.layout.flexlayout.FlexLayoutRenderer"
}));

sas.ui.cpb3.components.AComponentMixin.mixin(sas.risk.mrm.components.DimensionBrowserComponent);

sas.risk.mrm.components.DimensionBrowserComponent.prototype._createComponent=function(){
	this.getCpbForm().attachEvent('reload',$.proxy(this.onReload,this));
	this.dimNodeComp=new sas.risk.mrm.components.DimensionNodeComponent({
		cpbEnabled: true,
		cpbWidth: "100%",
		meta: this.getMeta(),
		headerPlugin: this.createActionsLayout(),
		select: $.proxy(this.handleDimNodeSelect,this)
	});
	this.previewLayout=this.createSelectionViewer();
	return this.createSplitter();
};
sas.risk.mrm.components.DimensionBrowserComponent.prototype.createActionsLayout=function(){
	var rb=sas.risk.mrm.util.MRMUtil.getBundle();
	this.createNewButton=new sas.risk.common.widgets.Button({
      press: $.proxy(this.actionCreateDimension,this),
      icon:sas.icons.HC.NEWSTARBURST,
      enabled: false
   });
	this.editButton=new sas.risk.common.widgets.Button({
      press: $.proxy(this.actionEditDimension,this),
      icon: sas.icons.HC.EDIT,
      enabled: false
   });
	this.moveButton=new sas.risk.common.widgets.Button({
      press: $.proxy(this.actionMoveDimension,this),
      icon: sas.icons.HC.MOVETO,
      enabled: false
   });
	this.deleteButton=new sas.risk.common.widgets.Button({
		press: $.proxy(this.actionDeleteDimension,this),
		icon: sas.icons.HC.DELETE,
		enabled: false
	});
	
	var actionsLayout = new sas.risk.common.layout.flexlayout.FlexLayout({
   	alignItems:sas.risk.common.layout.flexlayout.FlexLayoutAlignItems.Center,
      items: [this.createNewButton,this.editButton,this.moveButton,this.deleteButton]
   });
   return actionsLayout;
};
sas.risk.mrm.components.DimensionBrowserComponent.prototype.createSplitter=function(){
	var splitter=new sap.ui.commons.Splitter({
		splitterPosition:"50%",
		minSizeFirstPane:"20%",
		minSizeSecondPane:"20%"
	}).addStyleClass("MRMDimensionBrowser");
	splitter.addFirstPaneContent(this.dimNodeComp);
	splitter.addSecondPaneContent(this.previewLayout);
	return splitter;
};
sas.risk.mrm.components.DimensionBrowserComponent.prototype.createPlaceholder=function(){
	var rb=sas.risk.mrm.util.MRMUtil.getBundle();
	var placeHolderTxt=new sas.risk.common.layout.flexlayout.FlexLayout({
		width:"100%",
		height:"100%",
		justifyContent:sas.risk.common.layout.flexlayout.FlexLayoutJustifyContent.Center,
		alignItems:sas.risk.common.layout.flexlayout.FlexLayoutAlignItems.Center,
		items:[new sas.hc.m.Text({
			text: rb.getText("dimBrowser.preview.placeHolder.txt")
		})]
	});
	return placeHolderTxt;
};
sas.risk.mrm.components.DimensionBrowserComponent.prototype.createSelectionViewer=function(){
	var previewLayout=new sas.risk.common.layout.flexlayout.FlexLayout({
		width:"100%",
		height:"100%",
		items:[this.createPlaceholder()]
	}).addStyleClass("MRMDimPreview");
	return previewLayout;
};

//ACTIONS
sas.risk.mrm.components.DimensionBrowserComponent.prototype.handleDimNodeSelect=function(e){
	var that=this;
	this.actionRefreshCurrentSelection();
	
	this.selectedDimNode=e.getParameter("dimNode");
	this.selectedTreeNode=e.getParameter("treeNode");
	var isNotRoot=_.find(this.selectedDimNode.links,function(link){
		return link.rel==="parent";
	});
	if(this.selectedDimNode){
		this.editButton.setEnabled(true);
		this.createNewButton.setEnabled(true);
	}
	if(this.selectedDimNode && isNotRoot){
		this.moveButton.setEnabled(true);
		this.deleteButton.setEnabled(true);
	}
	//Fetch the screen context for any new selection to show the preview 
	var screenId=sas.risk.mrm.biz.DimensionDefinition[this.selectedDimNode.getDimensionName()].name;
	var collectionName=sas.risk.mrm.biz.DimensionDefinition[this.selectedDimNode.getDimensionName()].url;
	this.getCpbForm().getCpbAPI().loadObjectContext(screenId,collectionName,this.selectedDimNode.key,null)
	.done(function(json){
		json.cpb_data.SCREEN_READ_ONLY=true; //force the page to be readonly
		json.cpb_data.IS_SCREEN_PREVIEW=true;
		var cpbForm=new sas.ui.cpb3.CPBForm({
			screenId: screenId,
			data: sas.ui.cpb3.model.CpbObjectModel.bindData('/')
		});
		var cpbObjectModel=sas.ui.cpb3.model.CpbObjectModel.create(cpbForm);
		cpbObjectModel.refresh(json);
		that.previewLayout.destroyItems();
		that.previewLayout.addItem(cpbForm);
	});
};
sas.risk.mrm.components.DimensionBrowserComponent.prototype.actionRefreshCurrentSelection=function(){
	this.editButton.setEnabled(false);
	this.createNewButton.setEnabled(false);
	this.moveButton.setEnabled(false);
	this.deleteButton.setEnabled(false);
};
sas.risk.mrm.components.DimensionBrowserComponent.prototype.actionEditDimension=function(){
	var dimDef=sas.risk.mrm.biz.DimensionDefinition[this.selectedDimNode.getDimensionName()];
	sas.ui.cpb3.nav.NavUtil.getInstance().navTo(dimDef.url,{key: this.selectedDimNode.key});
};
sas.risk.mrm.components.DimensionBrowserComponent.prototype.actionMoveDimension=function(){
	var rb = sas.risk.mrm.util.MRMUtil.getBundle();
	var viewData={
		meta:this.getMeta(),
		selectedDimNode:this.selectedDimNode,
		selectedTreeNode:this.selectedTreeNode,
		cpbForm:this.previewLayout.getItems()[0],
		title:rb.getText("dimension.move.title.fmt.txt",[this.selectedDimNode.getDimensionName()])
	};
	sas.risk.mrm.util.DialogManager.open("MoveDimensionDialog",viewData,this.actionMoveRefresh,this);
};
sas.risk.mrm.components.DimensionBrowserComponent.prototype.actionRefresh=function(e){
	this.actionRefreshCurrentSelection(); //so that context is fetched again for any possible changes
	this.previewLayout.destroyItems();
	this.dimNodeComp.refreshTreeSelection(true);
};
sas.risk.mrm.components.DimensionBrowserComponent.prototype.actionMoveRefresh=function(e){ 
	var that=this;
	var rb=sas.risk.mrm.util.MRMUtil.getBundle();
	var dialog=e.getParameter("dialog");
	
	var currentDimNodeName=this.selectedDimNode.getDimensionName();
	var currentDimNodeKey=this.selectedDimNode.key;
	this.actionRefreshCurrentSelection();
	var hideInactive=this.dimNodeComp.getHideInactive();
	sas.risk.mrm.api.DimensionalLocationAPI.loadNodes(currentDimNodeName,[currentDimNodeKey],!hideInactive)
	.done(function(items){
		var i,item,dimNode;
		for(i=0;i<items.length;i++){
			item=items[i];
			dimNode=sas.risk.mrm.biz.DimNode.createFromJson(currentDimNodeName.toLowerCase(),item);
			sas.risk.mrm.biz.DimNodeRegistry.addToCache(dimNode);
			that.dimNodeComp.selectedDimNode=dimNode;
			that.previewLayout.destroyItems();
			that.dimNodeComp.refreshTreeSelection(true);
		}
		dialog.close();
	}).fail(function(){
		dialog.close();
	});
};
sas.risk.mrm.components.DimensionBrowserComponent.prototype.actionDeleteRefresh=function(e){ 
	var that=this;
	var rb=sas.risk.mrm.util.MRMUtil.getBundle();
	var changeReason= e ? e.getParameter("changeReason") : null;
	var dialog=e.getParameter("dialog");
	var deleteLink=_.find(this.selectedDimNode.links,function(link){return link.rel.toUpperCase()==="DELETE";});
	sas.ui.cpb3.api.CpbQueryAPI.getInstance().deleteQueryTableEntry(deleteLink.uri,deleteLink.method)
	.done(function(data){
		that.dimNodeComp.selectedDimNode=null; //remove selection in case of delete so the tree is refreshed right
		that.actionRefresh();
		that.previewLayout.addItem(that.createPlaceholder());
		dialog.close();
	}).fail(function(){
		dialog.close();
	});
	
};
sas.risk.mrm.components.DimensionBrowserComponent.prototype.actionCreateDimension=function(){
	var dimDef=sas.risk.mrm.biz.DimensionDefinition[this.selectedDimNode.getDimensionName()];
	//Save the parent path from currently selected Dim Node in the session storage to be picked up
	if(typeof(Storage)!==undefined){
		sessionStorage.setItem("init_parentPath",JSON.stringify(this.selectedDimNode.getHandle()));
	}else{
		console.warn("sessionStorage not supported by this browser");
	}
	sas.ui.cpb3.nav.NavUtil.getInstance().navTo(dimDef.url+"_drafts");
};
sas.risk.mrm.components.DimensionBrowserComponent.prototype.actionDeleteDimension=function(){
	var rb = sas.risk.mrm.util.MRMUtil.getBundle();
	var viewData={
		meta:this.getMeta(),
		selectedDimNode:this.selectedDimNode,
		cpbForm:this.previewLayout.getItems()[0],
		title:rb.getText("dimension.delete.title.fmt.txt",[this.selectedDimNode.getDimensionName(),this.selectedDimNode.getName()])
	};
	sas.risk.mrm.util.DialogManager.open("DeleteDimensionDialog",viewData,this.actionDeleteRefresh,this);
};

sas.risk.mrm.components.DimensionBrowserComponent.prototype.onReload = function(event) {
	var that=this;
	var key=event.getParameter("key");
	var type=event.getParameter("type");
	if(this.selectedDimNode !== undefined && sas.risk.mrm.biz.DimensionDefinition[this.selectedDimNode.getDimensionName()].url===type){
		if(this.selectedDimNode.key===key){//edit scenario
			this.actionRefresh();
		}else{//needed for create new scenario
			this.setBusy(true);
			var hideInactive=this.dimNodeComp.getHideInactive();
			sas.risk.mrm.api.DimensionalLocationAPI.loadNodes(this.selectedDimNode.getDimensionName(),[key],!hideInactive)
			.done(function(items){
				var i,item,dimNode;
				for(i=0;i<items.length;i++){
					item=items[i];
					dimNode=sas.risk.mrm.biz.DimNode.createFromJson(that.selectedDimNode.getDimensionName().toLowerCase(),item);
					sas.risk.mrm.biz.DimNodeRegistry.addToCache(dimNode);
					that.dimNodeComp.selectedDimNode=dimNode;
					that.actionRefresh();
				}
			}).always(function(e){
				 that.setBusy(false);
         });			
		}
	}
};

sas.risk.mrm.components.DimensionBrowserComponent.create=function(fieldJson, modelId, cpbForm) {
	var settings=sas.ui.cpb3.WidgetBindingUtil.getBasicSettings(fieldJson,modelId,cpbForm);
	return new sas.risk.mrm.components.DimensionBrowserComponent($.extend(
		settings,
		{
			cpbValue:sas.ui.cpb3.WidgetBindingUtil.getDataBindingPath(modelId,fieldJson.cpbName),
			meta : fieldJson.meta
		}
	));
};