jQuery.sap.declare("sas.risk.mrm.components.UserChooserComponent");
jQuery.sap.require("sas.ui.cpb3.components.AComponentMixin");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayout");
jQuery.sap.require("sas.risk.common.layout.flexlayout.FlexLayoutData");
jQuery.sap.require("sas.ui.cpb3.widget.InputField");
jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.hc.m.Dialog");
jQuery.sap.require("sas.risk.slimtable.widget.Item");
jQuery.sap.require("sas.risk.mrm.util.MRMUtil");
jQuery.sap.require("sas.risk.common.utils.CompositeUtil");
jQuery.sap.require("sas.risk.mrm.util.DialogManager");
jQuery.sap.require("sas.ui.cpb3.WidgetBindingUtil");
jQuery.sap.require("sas.risk.common.widgets.Button");

sas.risk.common.layout.flexlayout.FlexLayout.extend("sas.risk.mrm.components.UserChooserComponent", sas.ui.cpb3.components.AComponentMixin.mixinMeta({
    metadata:{
        properties:{
      	  "cpbLength":{type:"int"},
      	  "cpbMaxLength":{type:"int"},
      	  "roles":{type:"any"},
      	  "requiredCapability":{type:"any"},
      	  "locationFieldName":{type:"string"},
      	  "strictLocationMatching":{type:"boolean"},
      	  "activeOnly":{type:"boolean"},
      	  "location":{type:"string"},
      	  "meta":{type:"any"}
        },
        events:{
      	  openingDialog:{},
      	  closingDialog:{}
        }
    },
 	constructor:function(){
 		this.supportsLayout=true;
 		this.supportsLabel=true;
 		this.supportsAlign=true;
 		this.field = new sas.ui.cpb3.widget.InputField({
 	      showSuggestion: true,
 	      filterSuggests : false
 		});
 	   sas.risk.common.utils.CompositeUtil.addComposite(this,"cpbLength",this.field,"size");
 	   sas.risk.common.utils.CompositeUtil.addComposite(this,"cpbMaxLength",this.field,"maxLength");

 		sas.risk.common.layout.flexlayout.FlexLayout.apply(this,arguments);
 		this.setAlignItems(sas.risk.common.layout.flexlayout.FlexLayoutAlignItems.Baseline);
 		this.addStyleClass("cpbFieldLikeComponent");
 	},
 	renderer:"sas.risk.common.layout.flexlayout.FlexLayoutRenderer"
}));
sas.ui.cpb3.components.AComponentMixin.mixin(sas.risk.mrm.components.UserChooserComponent);

sas.risk.mrm.components.UserChooserComponent.prototype.setCpbValue=function(value){
	console.log("cpbValue = " + value);
	this.setProperty("cpbValue",value);
	var newFieldValue=value ? value.name : value;
	this.field.setValue(newFieldValue);
};

sas.risk.mrm.components.UserChooserComponent.prototype.setCpbEnabled=function(value){
	this.setProperty("cpbEnabled",value);
	this.field.setEnabled(value);
	if(value===false){this.field.setPlaceholder("");}
	this.chooser.setVisible(value);
};
sas.risk.mrm.components.UserChooserComponent.prototype.setCPBValue=function(value){
	this.setCpbValue(value);
};
sas.risk.mrm.components.UserChooserComponent.prototype.wireOnChange=function(onChangeFn){
	if(onChangeFn){
		this.field.attachChange(onChangeFn);
	}
};
sas.risk.mrm.components.UserChooserComponent.prototype._createComponent = function() {
   var rb = sas.risk.mrm.util.MRMUtil.getBundle();
   var that=this;
   
   var roles = this.getProperty("roles");
   var requiredCapability = this.getProperty("requiredCapability");
   var locationFieldName=this.getLocationFieldName().toUpperCase();
   this.bindProperty("location","cpb_data>/"+locationFieldName);
   
   this.field.setPlaceholder(rb.getText("userChooser.placeholder.txt"));
   this.field.attachSuggest(function(e){
      var v = e.getParameters().suggestValue;
      var f = e.getSource();
      
      var userUrl = sas.risk.mrm.components.UserChooserComponent.prototype._getUrl.call(that, v, roles, requiredCapability);
      
      $.getJSON(userUrl, $.proxy(function(userData){
         var data = userData.items;
         f.removeAllSuggestionItems();
         
         $.each(data, function(i, user){ 
            f.addSuggestionItem(new sas.risk.slimtable.widget.Item({
               key: user.key,
               text: user.displayNm + ' (' + user.userId + ')',
               payload: user
            }));
         });
      })
      );
   });
   this.field.attachSuggestionItemSelected(function(item){
   	console.log(item);
   	var user=item.mParameters.selectedItem.getPayload();
   	var result=sas.risk.mrm.util.MRMUtil.convertUserFromRestToUIContextFormat(user);  
   	that.setCpbValue(result);
   });
   //Attaching "attachSuggestionItemSelected" and "attachChange" both to the input field doesn't work as expected. In that case for suggestion selection also it reaches the attachChange, not the attachSuggestionItemSelected.
   //Therefore adding the attachSuggestionItemSelected logic and generic attachChange logic all in one "attachChange".
   this.field.attachChange(function(item){
   	if(that.field._oList && that.field.getSuggestionItems() && that.field.getSuggestionItems().length>0){ //If this change is coming from selecting a suggested item
   		if(that.field._oList.getSelectedItem()){ //selection through key press
   			that.field._fireSuggestionItemSelectedEvent(that.field._oList.getSelectedItem());
   			return;
   		}
   	    var selection=$("#"+that.field._oList.getId()+">ul>li:hover");//selection through mouse click
   	    //If user didn't select anything from the suggested items then we don't want to capture the user entered text as the value for user field
   	    if(!selection || selection.length===0){
   	        that.setCpbValue(null);
   	        return;
   	    }
   	    //Hack to get the suggestion selected item
   	    var selectedItemId=$("#"+that.field._oList.getId()+">ul>li:hover")[0].id;
   	    var suggestionItems=that.field.getSuggestionItems();
   	    var selectedSuggestionItem=_.find(suggestionItems,function(item){return selectedItemId.startsWith(item.getId());});
   	    var user=selectedSuggestionItem.getPayload();
   	    var result=sas.risk.mrm.util.MRMUtil.convertUserFromRestToUIContextFormat(user);
   	    that.setCpbValue(result);    
   	} else {
   		var newValue=item.getParameter("newValue");
   		if(newValue===""){//user blanked out the input field
   			that.setCpbValue(null);
   		}
   	}
   });
   
   var metaProp=this.getProperty("meta");
   
   var selectUserAction=function(e){
   	var user=e.getParameter("user");
   	if(user){
	   	var result=sas.risk.mrm.util.MRMUtil.convertUserFromRestToUIContextFormat(user);
	   	that.setCpbValue(result);
	   	that.field.removeAllSuggestionItems();
		   that.field.fireChange({value:result});
   	}
  	 	that.fireClosingDialog();
   };
   this.chooser = new sas.risk.common.widgets.Button({
	    icon: sas.icons.HC.USER,
	    press: function(e){
	   	 that.fireOpeningDialog();
	   	 var rb = sas.risk.mrm.util.MRMUtil.getBundle();
	   	 var filterUrl = that._getUrl.call(that, null, roles, requiredCapability);
	   	 var viewData={
   			 meta:metaProp,
   			 filterURL:filterUrl,
   			 title:rb.getText("userChooser.dialog.title")
			 };
   		 viewData.formattingKey=sas.risk.mrm.util.MRMUtil.buildDialogFormattingKey("UserChooserDialog",that.getCpbForm(),"UserChooserComponent",that.getCpbName());
	   	 sas.risk.mrm.util.DialogManager.open("UserChooserDialog",viewData,selectUserAction,that);
	    }
	});
   var layout=new sas.risk.common.layout.flexlayout.FlexLayout({
   	items:[this.field, this.chooser]
   });
   layout.addStyleClass("MRMChooser");
   return layout;
};

sas.risk.mrm.components.UserChooserComponent.prototype._getUrl = function(ids, roles, caps){
   var userUrl = "rest/users";
   var filters = null;
   var strict = this.getStrictLocationMatching();
   
   if (ids){
      var idFilter="or(contains(userId, \'###\'), contains(displayNm, \'###\'))".replace(/###/g, ids);
      filters = idFilter; 
   }
   
   if (roles){
   	roles=_.map(roles,function(role){return "'"+role+"'";});
   	var rolesJoin=roles.join();
      var roleFilter="roleNm(###)".replace(/###/g, rolesJoin);
      filters = filters ? "and(" + roleFilter + ", " + filters + ")" : roleFilter;
   }
   
   if (caps){
   	caps=_.map(caps,function(cap){return "'"+cap+"'";});
   	var capsJoin=caps.join();
      var capFilter="requiredCapability(###)".replace(/###/g, capsJoin);
      filters = filters ? "and(" + capFilter + ", " + filters + ")" : capFilter;
   }
   var loc=this.getLocation();
   if(loc){
   	var locationPoints=loc.split("|");
   	locationPoints.shift();
   	locationPoints.pop();
   	if (locationPoints.length > 0) {
   		var locFunc = strict ? "isUnderAll" : "isUnder";
   		var locFilter = locFunc+"("+locationPoints.join(",")+")";
   		filters = filters ? "and(" + locFilter + "," + filters + ")" : locFilter;   		
   	} 	
   }
   
   if (this.getActiveOnly()) {
      var activeFilter = "eq(activeFlg,true)";
   	filters = filters ? "and(" + activeFilter + ", " + filters + ")" : activeFilter;
   }
   
   if (filters){
      userUrl += "?" + $.param({filter:filters});
   }
   return userUrl;
};
//STATICS
sas.risk.mrm.components.UserChooserComponent.create=function(fieldJson, modelId, cpbForm) {
	var settings=sas.ui.cpb3.WidgetBindingUtil.getFieldBasicSettings(fieldJson,modelId,cpbForm);
	var lengthSettings=sas.ui.cpb3.WidgetBindingUtil.getLengthSettings(fieldJson,modelId,cpbForm);
	var userChooser = new sas.risk.mrm.components.UserChooserComponent($.extend(
		settings,
		lengthSettings,
		{
			cpbValue:sas.ui.cpb3.WidgetBindingUtil.getDataBindingPath(modelId,fieldJson.cpbName),
			roles : fieldJson.roles,
			requiredCapability : fieldJson.requiredCapabilities,
			locationFieldName : fieldJson.locationFieldName,
			strictLocationMatching : fieldJson.strictLocationMatching,
			activeOnly : fieldJson.activeOnly,
			meta : fieldJson.table_meta
		}
	));
	return userChooser;
};