jQuery.sap.require("sas.risk.mrm.widget.DimensionalLocation");
jQuery.sap.require("sas.risk.mrm.meta.formatter.FileSizeFormatter");
jQuery.sap.require("sas.risk.slimtable.util.RowActionUtil");
jQuery.sap.require("sas.risk.slimtable.widget.menu.AdvSearchItem");
jQuery.sap.require("sas.risk.mrm.widget.menu.ColumnsItem");
jQuery.sap.require("sas.risk.mrm.widget.menu.CustomizationInfoItem");
jQuery.sap.require("sas.risk.common.widgets.Button");
jQuery.sap.require("sas.risk.common.widgets.MessageDetailsBox");

sap.ui.jsview("sas.risk.mrm.view.dialog.ContentDeletedDialog", {
    getControllerName : function() {
        return "sas.risk.mrm.view.dialog.ContentDeletedDialog";
    },
    createContent : function(controller) {
        this.addStyleClass("mrmObjectViewPopup");
        var viewData=this.getViewData();
        var that=this;
        var rb = sas.risk.mrm.util.MRMUtil.getBundle();
        
        //FORMATTERS
        this.formatter = {"JAVA.LANG.LONG_fileSize":new sas.risk.mrm.meta.formatter.FileSizeFormatter("JAVA.LANG.LONG_fileSize")}; 
        sas.risk.slimtable.meta.ColumnFormatterRegistry.registerFactory(this.formatter,"ContentDeletedDialog");
        
        this.createButtons(controller);
        
        this.layout=new sas.risk.common.layout.flexlayout.FlexLayout({
            flexDirection:sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column,
            items:[]
        });
        var getTableMenuItems = function(){
     		var menuItems=[new sas.risk.slimtable.widget.menu.AdvSearchItem(),new sas.risk.mrm.widget.menu.ColumnsItem()];
     		var customizationInfoMeta=viewData.meta.tableMetaDeleted.customizationInfoMeta;
     		if(customizationInfoMeta){
     			var custInfoUrl="rest/utils/showCustomizationInfo/content/deleted/labelOverridePrefix/"+viewData.labelOverridePrefix;
     			if (customizationInfoMeta.fileName) {
     				custInfoUrl += "?fileName="+customizationInfoMeta.fileName;
     			}
     			menuItems.push(new sas.risk.mrm.widget.menu.CustomizationInfoItem({url:custInfoUrl,meta:customizationInfoMeta}));
     		}
     	   return menuItems;
       };
        this.table=new sas.risk.mrm.widget.MRMTable({
      	   formattingKey : viewData.formattingKey,
            data : $.proxy(this.fetchData, this),
            meta: viewData.meta.tableMetaDeleted,
            clientSide:true,
            showActionColumn:true,
            visibleRows: viewData.meta.tableMetaDeleted.dialogVisibleRows,
            selectionMode:sap.ui.table.SelectionMode.None,
            showSelectionControls:false,
            plugins:[new sas.risk.slimtable.plugins.Search(),
                     new sas.risk.slimtable.plugins.Filter(),
                     new sas.risk.slimtable.plugins.TableOptions({
                    	  menuItems: getTableMenuItems()
                      }),
                     new sas.risk.slimtable.plugins.Counter()
            ]
        });

        //ACTION COLUMN
        var actionColumn=sas.risk.slimtable.util.RowActionUtil.createActionColumn($.proxy(this.createActionMenuItems,this), this.table);
        this.table.addColumn(actionColumn);
        
        this.table.attachRowClick($.proxy(this.actionTableRowClicked,this));
        
        //When the table preference changes, update the local table meta json's column information accordingly        
        this.table.attachEvent("tablePreferenceChanged",function(event){
      	  var colNames=event.getParameter("rawColumnNames");
      	  sas.risk.slimtable.meta.TableMeta.updateTableMetaJsonUserDisplayColumns(viewData.meta.tableMetaDeleted,colNames);
        });
        
        this.layout.addItem(this.table);
        
        return this.layout;
    },
    createActionMenuItems : function(extra){
        var rb = sas.risk.mrm.util.MRMUtil.getBundle();
        var that=this;
        var rowData = extra.rowData;       
        var viewData=this.getViewData();
        
        var downloadItem = sas.risk.slimtable.util.RowActionUtil.createActionMenuItem({
           text : viewData.meta.downloadAction,
           callback : function(){that.actionRowDownload(this);}
        });
        
        var menuItems = [downloadItem];
        
        var historyItem = sas.risk.slimtable.util.RowActionUtil.createActionMenuItem({
           text : viewData.meta.viewHistoryAction,
           callback : function(){that.actionRowViewHistory(this);}
        });
        menuItems.push(historyItem);
        
        if (viewData.updateEnabled)
        {
           var restoreLink = sas.risk.mrm.api.ContentManagementAPI.findLink(rowData, "restore");
           var restoreItem = sas.risk.slimtable.util.RowActionUtil.createActionMenuItem({
              text : viewData.meta.restoreAction,
              callback : function(){that.actionRowRestore(this, restoreLink);}
           });
           menuItems.push(restoreItem);
        }
                
        return menuItems;
    },
    createButtons : function(controller) {
        var rb = sas.risk.mrm.util.MRMUtil.getBundle();
        var that=this;
        this.okButton = new sas.risk.common.widgets.Button({
            text : rb.getText("content.dialog.ok.txt"),
            press : $.proxy(controller.actionOk,controller),
        });
    },
    getDialogSettings : function() {
        var viewData=this.getViewData();
        return {
            beginButton : this.okButton,
            endButton : this.okButton,
            contentWidth:"50%"
        };
    },
    fetchData : function() {
       var that = this;
       var viewData = this.getViewData();
       
       this.getDialog().setBusyIndicatorDelay(0);
       this.getDialog().setBusy(true);
       
       var def = sas.risk.mrm.api.ContentManagementAPI.fetchDeletedAttachments(viewData.model,viewData.meta.groupNumber);
       def.done(function () {
          that.getDialog().setBusy(false);                      
       }).fail(function (e) {
      	 that.getDialog().setBusy(false);
          if (e.status && e.status === 404)
          {
             //The parent object no longer exists
             var errorJson = {"message" : viewData.meta.missingParentMsg};      
    	       sas.risk.common.widgets.MessageDetailsBox.error(errorJson, sas.config.appName);	
    	       that.getDialog().close();
          }
          else
          {
             sas.ui.cpb3.CpbUtil.handleFail(e);
          }
       });
       
       return def;
    },
    actionRowDownload : function(item) {
       var rowData = item.getParent().getPayload().rowData;
       if (rowData)
       {
          sas.risk.mrm.api.ContentManagementAPI.downloadAttachmentContent(rowData);
       }
    },
    actionTableRowClicked : function(e) {
       //CLICKING ON ROW WILL DOWNLOAD
       var rowData = e.getSource().getSource().getData()[e.getParameter("rowIdx")];
       sas.risk.mrm.api.ContentManagementAPI.downloadAttachmentContent(rowData);
    },
    actionRowViewHistory : function(item) {
       var rb = sas.risk.mrm.util.MRMUtil.getBundle();
       var that=this;
       var rowData = item.getParent().getPayload().rowData;
       var link = sas.risk.mrm.api.ContentManagementAPI.findLink(rowData, "history");
       if (link && link.href)
       {
      	 var formattingKey=_.map(that.getViewData().formattingKey,function(fk){
      		 return fk==="ContentDeletedDialog"?"ContentHistoryDialog":fk;
      	 });
          var viewData={
                meta:this.getController().getView().getViewData().meta,
                model:this.getController().getView().getViewData().model,
                historyLink:link,
                width:1000, //TODO improve
                height:475, 
                title:this.getController().getView().getViewData().meta.historyTitle,
                formattingKey:formattingKey
            };
          
          sas.risk.mrm.util.DialogManager.open("ContentHistoryDialog",viewData);
       }
    },
    actionRowRestore : function(item, restoreLink) {
       var rb = sas.risk.mrm.util.MRMUtil.getBundle();
       var that = this;
       var rowData = item.getParent().getPayload().rowData;
       var viewData = this.getViewData();
       
       if (restoreLink)
       {
          var callBackFn = function(action) {
             if(action==="CANCEL"){return;}
             that.getDialog().setBusyIndicatorDelay(0);
             that.getDialog().setBusy(true);
             sas.risk.mrm.api.ContentManagementAPI.restoreAttachment(viewData.model, viewData.meta.groupNumber, rowData, restoreLink).done(function(){
                 that.table.doRefresh();
                 that.getDialog().setBusy(false);
             }).fail(function(e){
            	 that.getDialog().setBusy(false);
                if (e.status && e.status === 404)
                {
                   //The parent object no longer exists
                   var errorJson = {"message" : viewData.meta.missingParentMsg};      
          	       sas.risk.common.widgets.MessageDetailsBox.error(errorJson, sas.config.appName);	
          	       that.getDialog().close();
                }
                else
                {
                   sas.ui.cpb3.CpbUtil.handleFail(e);
                }       	 
             });
         };
          
         var labelTxt =jQuery.sap.formatMessage(viewData.meta.restoreConfirmation, [rowData.fileName]); 
         var titleTxt = viewData.meta.restoreConfirmationTitle;
         sas.hc.m.MessageBox.warning(labelTxt, titleTxt, [ sas.hc.m.MessageBox.Action.OK, sas.hc.m.MessageBox.Action.CANCEL], null, callBackFn);
       } 
    },
});
sap.ui.controller("sas.risk.mrm.view.dialog.ContentDeletedDialog", {
    onInit:function(){
        var that=this;
        var viewData=this.getView().getViewData();
    },
    actionOk:function(){
       var dialog=this.getView().getDialog();
       dialog.setBusyIndicatorDelay(0);
       dialog.setBusy(true);
       this.getView().fireClosing({commentText:"test"});
       this.getView().getDialog().close();
    }
});