jQuery.sap.require("sas.ui.cpb3.CPBForm");
jQuery.sap.require("sas.risk.mrm.data.MRMObjectModel");
jQuery.sap.require("sas.risk.mrm.util.ContentDialogUtil");
jQuery.sap.require("sas.risk.mrm.api.MRMCpbObjectAPI");
jQuery.sap.require("sas.ui.cpb3.CpbUtil");
jQuery.sap.require("sas.risk.common.widgets.Button");

sap.ui.jsview("sas.risk.mrm.view.dialog.ContentHTMLDialog", {
   getControllerName : function() {
   	return "sas.risk.mrm.view.dialog.ContentHTMLDialog";
   },
   createContent : function(controller) {
      this.addStyleClass("mrmObjectViewPopup");
   	var viewData=this.getViewData();
   	var that=this;
   	var rb = sas.risk.mrm.util.MRMUtil.getBundle();
   	this.createButtons(controller);
   	this.cpbForm=new sas.ui.cpb3.CPBForm({
   		screenId: "Content",
   		data: sas.ui.cpb3.model.CpbObjectModel.bindData('/')
   	});
      var messages = new sas.risk.common.layout.flexlayout.FlexLayout({
   		width : "100%",
   		flexDirection : sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column,
   		items : {
   			path : sas.ui.cpb3.model.CpbObjectModel.bindMessagePath('/messages'),
   			factory : function(sId, context) {
   				var messageBar = new sas.risk.common.layout.flexlayout.FlexLayout({
   					items : [ 
   					         new sas.hc.m.Label({
   					         	text : sas.ui.cpb3.model.CpbObjectModel.bindMessage("message")
   					         }).addStyleClass("cpbMessage")
   					]
   				}).addStyleClass("cpbErrorMessageBar").addStyleClass("cpbMessageBar");
   				// possible add other possible types of message css classes here
   				// in future
   				return messageBar;
   			}
   		}
      });    
      this.layout=new sas.risk.common.layout.flexlayout.FlexLayout({
         flexDirection:sas.risk.common.layout.flexlayout.FlexLayoutDirection.Column,
         items:[
            messages,
            this.cpbForm
         ]
      });
      return this.layout;
   },
   createButtons : function(controller) {
      var rb = sas.risk.mrm.util.MRMUtil.getBundle();
      var that=this;
      var viewData = controller.getView().getViewData();
      if (viewData.create || viewData.update) {
      	this.okButton = new sas.risk.common.widgets.Button({
      	   text : rb.getText("content.dialog.ok.txt"),
      	   press : $.proxy(controller.actionOkCreateUpdate,controller),
      	});
      	this.cancelButton = new sas.risk.common.widgets.Button({
      		text : rb.getText("content.dialog.cancel.txt"),
      		press : $.proxy(controller.actionCancel,controller),
      	});
      }
      else {
         this.okButton = new sas.risk.common.widgets.Button({
         	text : rb.getText("content.dialog.ok.txt"),
         	press : $.proxy(controller.actionCancel,controller),
         });
      }
   },
   getDialogSettings : function() {
   	return {
	   	beginButton : this.okButton,
	   	endButton : this.cancelButton,
	   	contentWidth:"90%",
	   	contentHeight:"90%"
	   }; 
   },
   getCpbForm:function(){
   	return this.cpbForm;
   }
});
sap.ui.controller("sas.risk.mrm.view.dialog.ContentHTMLDialog", {
   onInit:function(){
      var view=this.getView();
		var cpbObjectModel=sas.ui.cpb3.model.CpbObjectModel.create();
		view.getCpbForm().setCpbObjectModel(cpbObjectModel);
      var viewData = view.getViewData();     
      var bo = viewData.model.getData("cpb_data")["BUSINESSOBJECT"];
      var rk = bo["handle"].split("|")[1];
      
      if (viewData.create)
      {
      	//initialize the ui context from the parent object
         var groupNo = viewData.meta.groupNumber;
      	
      	var overrides = {
            "BUSINESSOBJECTTYPE" : bo.classObject.businessObjectTypeName,
            "BUSINESSOBJECTRK" : new Number(rk),
            "PARENTBUSINESSOBJECT" : bo,
      		"ATTACHMENTTYPECD" : "CON",
      		"GROUPNUMBER" : groupNo,
      		"TEMP.HTMLCONTENT" : true
      	};       
      	view.getCpbForm().getCpbAPI().createObjectContext("Content","attachments",null,overrides).done(function(pageContext) {
      		cpbObjectModel.refresh(pageContext,view);
      	});
      }
      else
      {
      	//construct the UI context overrides from the selected attachment
         var att=view.getViewData().attachment;              
         var overrides = { "SCREEN_READ_ONLY" : !viewData.update,
            "BUSINESSOBJECTTYPE" : bo.classObject.businessObjectTypeName,
            "BUSINESSOBJECTRK" : new Number(rk),
            "PARENTBUSINESSOBJECT" : bo,
            "FILENAME" : att.fileName,
            "DESCRIPTION" : att.description,
            "REVISIONCD" : att.revisionCd,
            "CREATEDDTTM" : att.createdDttm,
            "MODIFIEDDTTM" : att.modifiedDttm,
            "VERSION" : att.versionNumber,
            "COMMENTS" : att.comment,
            "GROUPNUMBER" : att.groupNo,
            "FILE" : {fileName: att.fileName, fileSize: att.fileSize},
            "TEMP.HTMLCONTENT" : true
         };
         //if documet is locked by some other owner then the screen should be readonly
         if(att.lockOwner){
         	var isLockedByCurrentUser = sas.risk.mrm.api.ContentManagementAPI.isCurrentUserLockOwner(viewData.source.getModel("cpb_data"), att);
         	if(!isLockedByCurrentUser){
         		overrides["SCREEN_READ_ONLY"]=true;
         	}
         }
         
         //although these are users, they are strings in the ui definition
         var creatorLink = sas.risk.mrm.api.ContentManagementAPI.findLink(att, "creator");
         var updaterLink = sas.risk.mrm.api.ContentManagementAPI.findLink(att, "updater");
         var delegateLink = sas.risk.mrm.api.ContentManagementAPI.findLink(att, "delegate");
         if (creatorLink) overrides.CREATEDBY=creatorLink.title;
         if (updaterLink) overrides.MODIFIEDBY=updaterLink.title;
         if (delegateLink) overrides.DELEGATE=delegateLink.title;
         
         var link = sas.risk.mrm.api.ContentManagementAPI.findLink(att, "content");
         var d1=sas.risk.mrm.api.ContentManagementAPI.readHTMLContent(link.href);
         var d2=view.getCpbForm().getCpbAPI().loadObjectContext("Content","attachments",att.key, overrides);
         $.when(d1,d2).done(function(richContent, pageContext){
         	pageContext.cpb_data["TEMP.RICHTEXT"]=richContent; //The field name TEMP.richText is from the content.xml screen which is used to populate the content 
         	cpbObjectModel.refresh(pageContext,view);         	
         });
      }
   },
   actionOkCreateUpdate:function(){
   	var that = this;
   	var view = this.getView();
   	var cpbObjectModel=view.getCpbForm().getCpbObjectModel();
   	var viewData = view.getViewData();

   	view.getDialog().setBusyIndicatorDelay(0);
   	view.getDialog().setBusy(true);
      
   	var groupNo = viewData.meta.groupNumber;
   	var model = viewData.model;
      
   	//Find the filename from the content screen.  Get the rich text html from the richText component on the content screen.
   	var fileComponent = view.cpbForm.getCPBControl("fileName");
    	var fileId = fileComponent.getCpbId();
    	var richTextComponent = view.cpbForm.getCPBControl("TEMP.richText"); //The field name TEMP.richText is from the content.xml screen which is used to populate the content 
    	var newFileContent = richTextComponent.getCpbValue();
    	var newFile = newFileContent
    						? new Blob([newFileContent], { type: "text/xml"})
    						: new Blob([], {type:"text/xml"});
    
    	var fileTooBig = newFile && newFile.size > (viewData.meta.maxFileSizeMB * 1024 * 1024);
      
    	//Validate the UIContext before creating
    	var uiContext=cpbObjectModel.getPageContext(view).cpb_data;
    	uiContext.REQUIREDFIELDS=this._findRequiredControlIds(view);

    	var attId = uiContext["ATTACHMENTRK"];
    	view.getCpbForm().getCpbAPI().validateObjectContext("attachments",attId,uiContext,null).done(function (json) {
    		if (json.hasErrors || fileTooBig)
    		{
    			//Make sure the errors are put in the model, and the data is updated
    			view.getDialog().setBusy(false);
         
    			//Add a message to cover the large file case
    			if (fileTooBig)
    			{
    				json.hasErrors = true;
           
    				var rb = sas.risk.mrm.util.MRMUtil.getBundle();
    				var labelArgs=[viewData.meta.maxFileSizeMB];
    				var fileMsgTxt = jQuery.sap.formatMessage(that.getView().getViewData().meta.illegalFileSize, [labelArgs]);
           
    				var fileMsg = {message:fileMsgTxt, widgetId: fileId};
    				json.cpb_messages.messages = _.union(fileMsg, json.cpb_messages.messages);
    				json.cpb_messages.errored = _.union(fileId, json.cpb_messages.errored);
    			}
    			cpbObjectModel.applyErrors(json, view);
    		}
    		else
    		{           
    			var newAtt = that._buildAttachmentPayload(view, json.cpb_data);
    			var def = viewData.create ? sas.risk.mrm.api.ContentManagementAPI.createHTMLContent(model, groupNo, newAtt, newFile) :
    				sas.risk.mrm.api.ContentManagementAPI.updateHTMLContent(model, groupNo, newAtt, newFile);
    			def.done(function (data) {
    				viewData.source.fireChange();
    				view.fireClosing();
    				view.getDialog().close(); 
    				richTextComponent.fireEvent("destroyEditor");
    			})
    			.fail(function(e) {
    				view.getDialog().setBusy(false);
    				if (e.status && e.status === 400 && e.responseJSON.details)
    				{
    					//Add a message to cover the duplicate file case
    					var json = {hasErrors: true, cpb_data:uiContext, cpb_messages:that._buildMessagesForCpb(e)};
    					cpbObjectModel.applyErrors(json, view);
    				}
    			   else if (e.status && e.status === 404)
               {
                 //The parent object no longer exists
                 var errorJson = {"message" : viewData.meta.missingParentMsg};      
                 sas.risk.common.widgets.MessageDetailsBox.error(errorJson, sas.config.appName);	
               }
    				else
    				{
    					sas.ui.cpb3.CpbUtil.handleFail(e);
    				}
    			});
    		}
    	}).fail(function () {
    		view.getDialog().setBusy(false);
    	});
   },
   actionCancel:function() {
   	var richTextComponent = this.getView().cpbForm.getCPBControl("TEMP.richText");
   	if(richTextComponent){
   		richTextComponent.fireEvent("destroyEditor");
   	}
   	this.getView().getDialog().close();
   },
   _buildMessagesForCpb:function(e) {
   	var msgs = {errored:[], messages:[]};
   	var details = e.responseJSON.details;
   	for (var i = 0; i < details.length; i++)
   	{
   		msgs.errored.push(-1);
   		var msg = {message:details[i], widgetId: "-1"};
   		msgs.messages.push(msg);
   	}
   	return msgs;
   },
   _findRequiredControlIds:function(view) {
   	var requiredControls = _.filter(_.values(view.cpbForm._registryById),function(control){return control.getCpbRequired() && _.isNumber(control.getCpbId())});
   	return _.map(requiredControls, function(control){return control.getCpbId()});
   },
   _buildAttachmentPayload:function(view, cpb_data) {
   	if (view.getViewData().create)
   	{
   		var newAtt = {
   				typeCd : "CON",
   				groupNo : view.getViewData().meta.groupNumber,
   				fileName: cpb_data["FILENAME"] + ".html",
   				description: cpb_data["DESCRIPTION"],
   				revisionCd: cpb_data["REVISIONCD"],
   				comment: cpb_data["COMMENTS"],
   				customFields : sas.risk.mrm.util.ContentDialogUtil._buildCustomFieldsForRestPayload(view, cpb_data)
           	};
   		return newAtt;      
   	}
   	else
   	{
   		var existingAtt = view.getViewData().attachment;
   		var newAtt = JSON.parse(JSON.stringify( existingAtt ));
   		newAtt.fileName = cpb_data["FILENAME"];
   		newAtt.description = cpb_data["DESCRIPTION"];
   		newAtt.revisionCd = cpb_data["REVISIONCD"];
   		newAtt.comment = cpb_data["COMMENTS"];
   		newAtt.customFields = sas.risk.mrm.util.ContentDialogUtil._buildCustomFieldsForRestPayload(view, cpb_data);
   		return newAtt;      
   	}
   }
});