/******************************************************************************/
/*  Copyright (c) 2017 by SAS Institute, Inc., Cary, NC, USA.                 */
/*  All rights reserved.                                                      */
/******************************************************************************/
/*  NAME:        rskmgt_get_grid_props.sas                                    */
/*  VERSION:     7.3                                                          */
/*  DESCRIPTION: Sets up the global macros needed for accessing the GRID env. */
/*  PRODUCT:     SAS Risk Management Server                                   */
/******************************************************************************/

%macro rskmgt_get_grid_props;

%put NOTE: {START: &sysmacroname}; 

  /* Read the grid_properties.txt file to get all connection information to */
  /* connect to the grid environment. */
  %if %sysfunc(exist(&prod.cntl.grid_properties))=0 %then 
  %do;
    data &prod.cntl.grid_properties;
      infile CNTLD(grid_properties.txt) dlm=',' firstobs=2;
      length property $20;
      length value $300;
      input property value;
    run;
  %end;  
  
  /* Assign a temp filename to write the SAS code to a file */
  filename tmppgm temp;
  
  /* Read the &prod.cntl.grid_properties file to lookup the environment */
  /* info needed to connect to the grid */
  data _null_;
    length prop_file prop_map $32 type $300;
    set &prod.cntl.grid_properties;

   prop_file=strip(property)||'.xml';
   prop_map=strip(property)||'.map';
   type=scan(value,1,'[');
    
    file tmppgm;
    
    put '/* Assign output XML file to a work location */';
    put '%let outputFile=%trim(%sysfunc(pathname(work,l))/' prop_file ');';
    put '/* Assign the xml file to a temporary work location */';
    put 'filename gridin temp;';
    put 'filename gridout "&outputFile";';
    put;
    put 'data _null_;';
    put 'file gridin;';
    put 'input;';
    put "put _infile_ ' ';";
    put 'datalines;';
    put '<GetMetadataObjects>';
    put '<Reposid>$METAREPOSITORY</Reposid>';
    put '<Type>';
    put type;
    put '</Type>';
    put '<NS>SAS</NS>';
    put '<Flags>396</Flags>';
    put '<Options>';
    put '<XMLSelect search="' value '"/>';
    put '</Options>';
    put '</GetMetadataObjects>';
    put ';';
    put 'run;';
    put;
    put 'proc metadata in=gridin out=gridout;';
    put 'run;';
    put;
    put '/* Filename to the map file */';
    put 'filename dimmap "&&&prod._control_file_path./' prop_map '";';
    put '/* XML libname used to read the xml file */';
    put 'libname svc_rsp xmlv2 "&outputFile" xmlmap=dimmap access=readonly;';
    put;
    put '/* Copy the xml files to the work area */';
    put 'proc copy in=svc_rsp out=work;';
    put 'run;';
    put;
    put '/* Clear the filename and libname statements */';
    put 'libname svc_rsp clear;';
    put 'filename dimmap clear;';
  run;
       
  %include tmppgm;
  filename tmppgm clear;
  
  /* Read the output datasets and put the grid property values into macro variables */
  data _null_;
    set work.grid_host;
    call symputx('grid_host',Grid_Prop_DefVal);
  run;
  
  data _null_;
    set work.grid_port;
    call symputx('grid_port',connect_port);
  run;
  
  data _null_;
    set work.gridmode;
    
    if Grid_Prop_DefVal=1 then call symput('grid_mode','NON-DISTRIBUTED');
    else if Grid_Prop_DefVal=0 then call symput('grid_mode','DISTRIBUTED');
  run;
  
  data _null_;
    set work.gridlib;
    call symput('lasr_meta_lib',grid_lib_name);
  run;
    
  /* Get the lasr signer URL */
  data _null_;
    length uri $256 value $32767;
    uri='';
    value='';

    LibUri = "omsobj:SASLibrary?@Name='" || "&lasr_meta_lib" || "'";

    rc = metadata_getnasn(LibUri, "LibraryConnection", 1, uri);
    if rc >= 0; 

    rc = metadata_getprop(uri,"Connection.LASR.PropertyGroup.Signer.xmlKey.txt", value, uri);
    if rc >= 0 then call symput('lasr_signer_url',strip(value));
  run;
    
  %put NOTE: {END: &sysmacroname}; 

%mend rskmgt_get_grid_props;